@@ -1,11 +1,11 @@
 # pylint:disable=unused-argument,arguments-differ
-from typing import Set, Union
+from __future__ import annotations
 import logging
 
-import ailment
+import angr.ailment as ailment
 
-from ..sequence_walker import SequenceWalker
-from ..structuring.structurer_nodes import (
+from angr.analyses.decompiler.sequence_walker import SequenceWalker
+from angr.analyses.decompiler.structuring.structurer_nodes import (
     SequenceNode,
     CodeNode,
     MultiNode,
@@ -14,7 +14,7 @@ from ..structuring.structurer_nodes import (
     CascadingConditionNode,
 )
 from .node_address_finder import NodeAddressFinder
-from ....knowledge_plugins.gotos import Goto
+from angr.analyses.decompiler.goto_manager import Goto
 
 
 l = logging.getLogger(name=__name__)
@@ -44,9 +44,10 @@ class GotoSimplifier(SequenceWalker):
         }
         self._function = function
         self._kb = kb
+        self.irreducible_gotos = set()
 
         super().__init__(handlers)
-        self._node_addrs: Set[int] = NodeAddressFinder(node).addrs
+        self._node_addrs: set[int] = NodeAddressFinder(node).addrs
 
         self.walk(node)
 
@@ -57,7 +58,7 @@ class GotoSimplifier(SequenceWalker):
         :return:
         """
 
-        for n0, n1 in zip(node.nodes, node.nodes[1:] + [successor]):
+        for n0, n1 in zip(node.nodes, [*node.nodes[1:], successor]):
             self._handle(n0, successor=n1)
 
     def _handle_codenode(self, node, successor=None, **kwargs):
@@ -108,7 +109,7 @@ class GotoSimplifier(SequenceWalker):
         :return:
         """
 
-        for n0, n1 in zip(node.nodes, node.nodes[1:] + [successor]):
+        for n0, n1 in zip(node.nodes, [*node.nodes[1:], successor]):
             self._handle(n0, successor=n1)
 
     def _handle_block(self, block, successor=None, **kwargs):  # pylint:disable=no-self-use
@@ -140,29 +141,38 @@ class GotoSimplifier(SequenceWalker):
                     block.statements = block.statements[:-1]
         # if {goto label_1;} else {goto label_2;}
         elif isinstance(last_stmt, ailment.Stmt.ConditionalJump):
-            if last_stmt.true_target and isinstance(last_stmt.true_target.value, int):
+            if (
+                last_stmt.true_target
+                and isinstance(last_stmt.true_target, ailment.Expr.Const)
+                and isinstance(last_stmt.true_target.value, int)
+            ):
                 self._handle_irreducible_goto(block, last_stmt, branch_target=True)
 
-            if last_stmt.false_target and isinstance(last_stmt.false_target.value, int):
+            if (
+                last_stmt.false_target
+                and isinstance(last_stmt.false_target, ailment.Expr.Const)
+                and isinstance(last_stmt.false_target.value, int)
+            ):
                 self._handle_irreducible_goto(block, last_stmt, branch_target=False)
 
     def _handle_irreducible_goto(
-        self, block, goto_stmt: Union[ailment.Stmt.Jump, ailment.Stmt.ConditionalJump], branch_target=None
+        self, block, goto_stmt: ailment.Stmt.Jump | ailment.Stmt.ConditionalJump, branch_target=None
     ):
-        if not self._kb or not self._function:
+        if not self._function:
             l.debug("Unable to store a goto at %#x because simplifier is kb or functionless", block.addr)
             return
 
         # normal Goto Label
         if branch_target is None:
-            stmt_target = goto_stmt.target
+            dst_target = goto_stmt.target
         # true branch of a conditional jump
         elif branch_target:
-            stmt_target = goto_stmt.true_target
+            dst_target = goto_stmt.true_target
         # false branch of a conditional jump
         else:
-            stmt_target = goto_stmt.true_target
+            dst_target = goto_stmt.false_target
 
-        goto = Goto(addr=goto_stmt.ins_addr or block.addr, target_addr=stmt_target.value)
-        l.debug("Storing %r in kb.gotos", goto)
-        self._kb.gotos.locations[self._function.addr].add(goto)
+        src_ins_addr = goto_stmt.ins_addr if "ins_addr" in goto_stmt.tags else block.addr
+        goto = Goto(block.addr, dst_target.value, src_idx=block.idx, dst_idx=None, src_ins_addr=src_ins_addr)
+        l.debug("Storing %r goto", goto)
+        self.irreducible_gotos.add(goto)
