@@ -1,25 +1,26 @@
+from __future__ import annotations
 import operator
-from typing import Callable, Iterable, Tuple
+from collections.abc import Callable, Iterable
 
-from pypcode import OpCode
 import claripy
 from claripy.ast.bv import BV
+from pypcode import OpCode
+
+from angr.errors import AngrError
 
-from ...errors import AngrError
 
 # pylint:disable=abstract-method
 
 
-def make_bv_sizes_equal(bv1: BV, bv2: BV) -> Tuple[BV, BV]:
+def make_bv_sizes_equal(bv1: BV, bv2: BV) -> tuple[BV, BV]:
     """
     Makes two BVs equal in length through sign extension.
     """
     if bv1.size() < bv2.size():
         return (bv1.sign_extend(bv2.size() - bv1.size()), bv2)
-    elif bv1.size() > bv2.size():
+    if bv1.size() > bv2.size():
         return (bv1, bv2.sign_extend(bv1.size() - bv2.size()))
-    else:
-        return (bv1, bv2)
+    return (bv1, bv2)
 
 
 # FIXME: Unimplemented ops (mostly floating point related) have associated C++
@@ -31,7 +32,7 @@ class OpBehavior:
     Base class for all operation behaviors.
     """
 
-    __slots__ = ("opcode", "is_unary", "is_special")
+    __slots__ = ("is_special", "is_unary", "opcode")
     opcode: int
     is_unary: bool
     is_special: bool
@@ -864,6 +865,8 @@ class OpBehaviorSubpiece(OpBehavior):
     def evaluate_binary(self, size_out: int, size_in: int, in1: BV, in2: BV) -> BV:
         if in2.size() < in1.size():
             in2 = in2.sign_extend(in1.size() - in2.size())
+        if in1.size() < in2.size():
+            in1 = in1.sign_extend(in2.size() - in1.size())
         return (in1 >> (in2 * 8)) & (2 ** (size_out * 8) - 1)
 
 
@@ -882,6 +885,23 @@ class OpBehaviorPopcount(OpBehavior):
         return expr
 
 
+class OpBehaviorLzcount(OpBehavior):
+    """
+    Behavior for the LZCOUNT operation.
+    """
+
+    def __init__(self):
+        super().__init__(OpCode.LZCOUNT, True)
+
+    def evaluate_unary(self, size_out: int, size_in: int, in1: BV) -> BV:
+        expr = claripy.BVV(len(in1), size_out * 8)
+        for pos in range(len(in1)):
+            expr = claripy.If(
+                claripy.Extract(pos, pos, in1) == claripy.BVV(1, 1), claripy.BVV(len(in1) - pos - 1, size_out * 8), expr
+            )
+        return expr
+
+
 class BehaviorFactory:
     """
     Returns the behavior object for a given opcode.
@@ -889,7 +909,8 @@ class BehaviorFactory:
 
     def __init__(self):
         self._behaviors = {}
-        self._register_behaviors()
+        if OpCode:
+            self._register_behaviors()
 
     def get_behavior_for_opcode(self, opcode: int) -> OpBehavior:
         return self._behaviors[opcode]
@@ -968,5 +989,6 @@ class BehaviorFactory:
                 OpCode.INSERT: OpBehavior(OpCode.INSERT, False, True),
                 OpCode.EXTRACT: OpBehavior(OpCode.EXTRACT, False, True),
                 OpCode.POPCOUNT: OpBehaviorPopcount(),
+                OpCode.LZCOUNT: OpBehaviorLzcount(),
             }
         )
