@@ -1,23 +1,26 @@
-from typing import Optional, Iterable, Dict, Set, Generator, Tuple, Union, Any, TYPE_CHECKING, overload, Type
+from __future__ import annotations
+from typing import Any, TYPE_CHECKING, cast, overload
+from collections import defaultdict
+from collections.abc import Iterable, Generator
+from enum import Enum, auto
 import weakref
 import logging
-from enum import Enum, auto
-
-from collections import defaultdict
 
 import claripy
 from claripy.annotation import Annotation
 import archinfo
 
-from angr.misc.ux import deprecated
 from angr.errors import SimMemoryMissingError, SimMemoryError
 from angr.storage.memory_mixins import MultiValuedMemory
-from angr.storage.memory_mixins.paged_memory.pages.multi_values import MultiValues
+from angr.storage.memory_mixins.paged_memory.pages.multi_values import MVType, MultiValues
+from angr.knowledge_plugins.key_definitions.definition import A
 from angr.engines.light import SpOffset
 from angr.code_location import CodeLocation, ExternalCodeLocation
+from angr.utils.constants import is_alignment_mask
 from .atoms import Atom, Register, MemoryLocation, Tmp, ConstantSrc
 from .definition import Definition, Tag
 from .heap_address import HeapAddress
+from .undefined import Undefined
 from .uses import Uses
 
 if TYPE_CHECKING:
@@ -49,7 +52,7 @@ class DefinitionAnnotation(Annotation):
     An annotation that attaches a `Definition` to an AST.
     """
 
-    __slots__ = ("definition", "_hash")
+    __slots__ = ("_hash", "definition")
 
     def __init__(self, definition):
         super().__init__()
@@ -67,14 +70,13 @@ class DefinitionAnnotation(Annotation):
     def __hash__(self):
         return self._hash
 
-    def __eq__(self, other: "object"):
+    def __eq__(self, other: object):
         if type(other) is DefinitionAnnotation:
             return self.definition == other.definition
-        else:
-            return False
+        return False
 
     def __repr__(self):
-        return f"<{self.__class__.__name__}({repr(self.definition)})"
+        return f"<{self.__class__.__name__}({self.definition!r})"
 
 
 # pylint: disable=W1116
@@ -89,24 +91,24 @@ class LiveDefinitions:
     _tops = {}
 
     __slots__ = (
-        "project",
+        "__weakref__",
+        "_canonical_size",
         "arch",
-        "track_tmps",
+        "heap",
+        "heap_uses",
+        "memory",
+        "memory_uses",
+        "other_uses",
+        "others",
+        "project",
+        "register_uses",
         "registers",
         "stack",
-        "heap",
-        "memory",
-        "tmps",
-        "others",
-        "other_uses",
-        "register_uses",
         "stack_uses",
-        "heap_uses",
-        "memory_uses",
-        "uses_by_codeloc",
         "tmp_uses",
-        "_canonical_size",
-        "__weakref__",
+        "tmps",
+        "track_tmps",
+        "uses_by_codeloc",
     )
 
     def __init__(
@@ -127,8 +129,9 @@ class LiveDefinitions:
         tmp_uses=None,
         other_uses=None,
         element_limit=5,
+        merge_into_tops: bool = True,
     ):
-        self.project: Optional["Project"] = None
+        self.project: Project | None = None
         self.arch = arch
         self.track_tmps = track_tmps
         self._canonical_size: int = canonical_size  # TODO: Drop canonical_size
@@ -142,6 +145,7 @@ class LiveDefinitions:
                 page_kwargs={"mo_cmp": self._mo_cmp},
                 endness=self.arch.register_endness,
                 element_limit=element_limit,
+                merge_into_top=merge_into_tops,
             )
             if registers is None
             else registers
@@ -154,6 +158,7 @@ class LiveDefinitions:
                 skip_missing_values_during_merging=False,
                 page_kwargs={"mo_cmp": self._mo_cmp},
                 element_limit=element_limit,
+                merge_into_top=merge_into_tops,
             )
             if stack is None
             else stack
@@ -166,6 +171,7 @@ class LiveDefinitions:
                 skip_missing_values_during_merging=False,
                 page_kwargs={"mo_cmp": self._mo_cmp},
                 element_limit=element_limit,
+                merge_into_top=merge_into_tops,
             )
             if memory is None
             else memory
@@ -178,12 +184,13 @@ class LiveDefinitions:
                 skip_missing_values_during_merging=False,
                 page_kwargs={"mo_cmp": self._mo_cmp},
                 element_limit=element_limit,
+                merge_into_top=merge_into_tops,
             )
             if heap is None
             else heap
         )
-        self.tmps: Dict[int, Set[Definition]] = {} if tmps is None else tmps
-        self.others: Dict[Atom, MultiValues] = others if others is not None else {}
+        self.tmps: dict[int, set[Definition]] = {} if tmps is None else tmps
+        self.others: dict[Atom, MultiValues] = others if others is not None else {}
 
         # set state
         self.registers.set_state(self)
@@ -195,38 +202,18 @@ class LiveDefinitions:
         self.stack_uses = Uses() if stack_uses is None else stack_uses
         self.heap_uses = Uses() if heap_uses is None else heap_uses
         self.memory_uses = Uses() if memory_uses is None else memory_uses
-        self.tmp_uses: Dict[int, Set[CodeLocation]] = defaultdict(set) if tmp_uses is None else tmp_uses
+        self.tmp_uses: dict[int, set[CodeLocation]] = defaultdict(set) if tmp_uses is None else tmp_uses
         self.other_uses = Uses() if other_uses is None else other_uses
 
-        self.uses_by_codeloc: Dict[CodeLocation, Set[Definition]] = defaultdict(set)
-
-    @property
-    @deprecated("registers")
-    def register_definitions(self) -> MultiValuedMemory:
-        return self.registers
-
-    @property
-    @deprecated("stack")
-    def stack_definitions(self) -> MultiValuedMemory:
-        return self.stack
-
-    @property
-    @deprecated("memory")
-    def memory_definitions(self) -> MultiValuedMemory:
-        return self.memory
-
-    @property
-    @deprecated("heap")
-    def heap_definitions(self) -> MultiValuedMemory:
-        return self.heap
+        self.uses_by_codeloc: dict[CodeLocation, set[Definition]] = defaultdict(set)
 
     def __repr__(self):
         ctnt = "LiveDefs"
         if self.tmps:
-            ctnt += ", %d tmpdefs" % len(self.tmps)
-        return "<%s>" % ctnt
+            ctnt += f", {len(self.tmps)} tmpdefs"
+        return f"<{ctnt}>"
 
-    def copy(self, discard_tmpdefs=False) -> "LiveDefinitions":
+    def copy(self, discard_tmpdefs=False) -> LiveDefinitions:
         rd = LiveDefinitions(
             self.arch,
             track_tmps=self.track_tmps,
@@ -260,8 +247,8 @@ class LiveDefinitions:
 
     @staticmethod
     def _mo_cmp(
-        mo_self: Union["SimMemoryObject", Set["SimMemoryObject"]],
-        mo_other: Union["SimMemoryObject", Set["SimMemoryObject"]],
+        mo_self: SimMemoryObject | set[SimMemoryObject],
+        mo_other: SimMemoryObject | set[SimMemoryObject],
         addr: int,
         size: int,
     ):  # pylint:disable=unused-argument
@@ -317,7 +304,7 @@ class LiveDefinitions:
                 return True
         return False
 
-    def stack_address(self, offset: int) -> Optional[claripy.ast.bv.BV]:
+    def stack_address(self, offset: int) -> claripy.ast.bv.BV:
         base = claripy.BVS("stack_base", self.arch.bits, explicit_name=True)
         if offset:
             return base + offset
@@ -328,7 +315,7 @@ class LiveDefinitions:
         return "stack_base" in addr.variables
 
     @staticmethod
-    def get_stack_offset(addr: claripy.ast.Base, had_stack_base=False) -> Optional[int]:
+    def get_stack_offset(addr: claripy.ast.Base, had_stack_base=False) -> int | None:
         if had_stack_base and addr.op == "BVV":
             assert isinstance(addr, claripy.ast.BV)
             return addr.concrete_value
@@ -337,23 +324,36 @@ class LiveDefinitions:
         if "stack_base" in addr.variables:
             if addr.op == "BVS":
                 return 0
-            elif addr.op == "__add__":
+            if addr.op == "__add__":
                 if len(addr.args) == 2:
-                    off0 = LiveDefinitions.get_stack_offset(addr.args[0], had_stack_base=True)
-                    off1 = LiveDefinitions.get_stack_offset(addr.args[1], had_stack_base=True)
+                    off0 = LiveDefinitions.get_stack_offset(cast(claripy.ast.BV, addr.args[0]), had_stack_base=True)
+                    off1 = LiveDefinitions.get_stack_offset(cast(claripy.ast.BV, addr.args[1]), had_stack_base=True)
                     if off0 is not None and off1 is not None:
                         return off0 + off1
                 elif len(addr.args) == 1:
                     return 0
             elif addr.op == "__sub__" and len(addr.args) == 2:
-                off0 = LiveDefinitions.get_stack_offset(addr.args[0], had_stack_base=True)
-                off1 = LiveDefinitions.get_stack_offset(addr.args[1], had_stack_base=True)
+                off0 = LiveDefinitions.get_stack_offset(cast(claripy.ast.BV, addr.args[0]), had_stack_base=True)
+                off1 = LiveDefinitions.get_stack_offset(cast(claripy.ast.BV, addr.args[1]), had_stack_base=True)
                 if off0 is not None and off1 is not None:
                     return off0 - off1
         return None
 
     @staticmethod
-    def annotate_with_def(symvar: claripy.ast.BV, definition: Definition) -> claripy.ast.BV:
+    def _simplify_sp_alignment(data: MultiValues) -> MultiValues:
+        """
+        Eliminate trivial stack pointer alignment, e.g.: {sp & ~0xf} -> {sp}
+        """
+        value = data.one_value()
+        if value is not None and "stack_base" in value.variables and value.op == "__and__" and len(value.args) == 2:
+            if value.args[1].concrete and is_alignment_mask(value.args[1].concrete_value):
+                return MultiValues(value.args[0])
+            if value.args[0].concrete and is_alignment_mask(value.args[0].concrete_value):
+                return MultiValues(value.args[1])
+        return data
+
+    @staticmethod
+    def annotate_with_def(symvar: MVType, definition: Definition) -> MVType:
         """
 
         :param symvar:
@@ -371,13 +371,13 @@ class LiveDefinitions:
         return symvar.annotate(DefinitionAnnotation(definition), remove_annotations=annotations_to_remove)
 
     @staticmethod
-    def extract_defs(symvar: claripy.ast.Base) -> Generator[Definition, None, None]:
+    def extract_defs(symvar: claripy.ast.Base) -> Generator[Definition]:
         for anno in symvar.annotations:
             if isinstance(anno, DefinitionAnnotation):
                 yield anno.definition
 
     @staticmethod
-    def extract_defs_from_annotations(annos: Iterable["Annotation"]) -> Set[Definition]:
+    def extract_defs_from_annotations(annos: Iterable[Annotation]) -> set[Definition]:
         defs = set()
         for anno in annos:
             if isinstance(anno, DefinitionAnnotation):
@@ -385,7 +385,7 @@ class LiveDefinitions:
         return defs
 
     @staticmethod
-    def extract_defs_from_mv(mv: MultiValues) -> Generator[Definition, None, None]:
+    def extract_defs_from_mv(mv: MultiValues) -> Generator[Definition]:
         for vs in mv.values():
             for v in vs:
                 yield from LiveDefinitions.extract_defs(v)
@@ -410,7 +410,7 @@ class LiveDefinitions:
         assert result is not None
         return result
 
-    def get_sp_offset(self) -> Optional[int]:
+    def get_sp_offset(self) -> int | None:
         """
         Return the offset of the stack pointer.
         """
@@ -419,14 +419,13 @@ class LiveDefinitions:
         sp_v = sp_values.one_value()
         if sp_v is None:
             values = [v for v in next(iter(sp_values.values())) if self.get_stack_offset(v) is not None]
-            assert len({self.get_stack_offset(v) for v in values}) == 1
-            result = self.get_stack_offset(values[0])
-            return result
+            if len({self.get_stack_offset(v) for v in values}) != 1:
+                return None
+            return self.get_stack_offset(values[0])
 
-        result = self.get_stack_offset(sp_v)
-        return result
+        return self.get_stack_offset(sp_v)
 
-    def get_stack_address(self, offset: claripy.ast.Base) -> Optional[int]:
+    def get_stack_address(self, offset: claripy.ast.Base) -> int | None:
         offset_int = self.get_stack_offset(offset)
         if offset_int is None:
             return None
@@ -440,10 +439,10 @@ class LiveDefinitions:
             base_v = self.INITIAL_SP_64BIT
             mask = 0xFFFF_FFFF_FFFF_FFFF
         else:
-            raise ValueError("Unsupported architecture word size %d" % self.arch.bits)
+            raise ValueError(f"Unsupported architecture word size {self.arch.bits}")
         return (base_v + offset) & mask
 
-    def merge(self, *others: "LiveDefinitions") -> Tuple["LiveDefinitions", bool]:
+    def merge(self, *others: LiveDefinitions) -> tuple[LiveDefinitions, bool]:
         state = self.copy()
 
         merge_occurred = state.registers.merge([other.registers for other in others], None)
@@ -470,7 +469,7 @@ class LiveDefinitions:
 
         return state, merge_occurred
 
-    def compare(self, other: "LiveDefinitions") -> bool:
+    def compare(self, other: LiveDefinitions) -> bool:
         r0 = self.registers.compare(other.registers)
         if r0 is False:
             return False
@@ -516,7 +515,8 @@ class LiveDefinitions:
                 else:
                     l.warning("Skip stack storing since the stack offset is None.")
             elif isinstance(atom.addr, HeapAddress):
-                self.heap.erase(atom.addr.value, size=atom.size)
+                if not isinstance(atom.addr.value, Undefined):
+                    self.heap.erase(atom.addr.value, size=atom.size)
             elif isinstance(atom.addr, int):
                 self.memory.erase(atom.addr, size=atom.size)
             elif isinstance(atom.addr, claripy.ast.Base):
@@ -545,10 +545,10 @@ class LiveDefinitions:
         code_loc: CodeLocation,
         data: MultiValues,
         dummy=False,
-        tags: Optional[Set[Tag]] = None,
+        tags: set[Tag] | None = None,
         endness=None,
         annotated=False,
-    ) -> Optional[MultiValues]:
+    ) -> MultiValues | None:
         if data is None:
             raise TypeError("kill_and_add_definition() does not take None as data.")
 
@@ -557,12 +557,20 @@ class LiveDefinitions:
         else:
             definition: Definition = Definition(atom, code_loc, dummy=dummy, tags=tags)
             d = MultiValues()
+            count = 0
             for offset, vs in data.items():
                 for v in vs:
+                    count += 1
                     d.add_value(offset, self.annotate_with_def(v, definition))
+            assert count != 0, "MV may not be empty, use TOP instead"
 
         # set_object() replaces kill (not implemented) and add (add) in one step
         if isinstance(atom, Register):
+
+            # Tolerate simple stack pointer alignments
+            if atom.reg_offset == self.arch.sp_offset:
+                d = self._simplify_sp_alignment(d)
+
             try:
                 self.registers.store(
                     atom.reg_offset,
@@ -603,7 +611,7 @@ class LiveDefinitions:
                 return None
         elif isinstance(atom, Tmp):
             if self.track_tmps:
-                self.tmps[atom.tmp_idx] = {definition}
+                self.tmps[atom.tmp_idx] = {Definition(atom, code_loc, dummy=dummy, tags=tags)}
             else:
                 self.tmps[atom.tmp_idx] = self.uses_by_codeloc[code_loc]
                 return None
@@ -612,7 +620,7 @@ class LiveDefinitions:
 
         return d
 
-    def add_use(self, atom: Atom, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_use(self, atom: Atom, code_loc: CodeLocation, expr: Any | None = None) -> None:
         if isinstance(atom, Register):
             self.add_register_use(atom.reg_offset, atom.size, code_loc, expr=expr)
         elif isinstance(atom, MemoryLocation):
@@ -654,15 +662,15 @@ class LiveDefinitions:
             self.other_uses.add_use(definition, code_loc, expr)
 
     def get_definitions(
-        self, thing: Union[Atom, Definition[Atom], Iterable[Atom], Iterable[Definition[Atom]], MultiValues]
-    ) -> Set[Definition[Atom]]:
+        self, thing: Atom | Definition[Atom] | Iterable[Atom] | Iterable[Definition[Atom]] | MultiValues
+    ) -> set[Definition[Atom]]:
         if isinstance(thing, MultiValues):
             defs = set()
             for vs in thing.values():
                 for v in vs:
                     defs.update(LiveDefinitions.extract_defs_from_annotations(v.annotations))
             return defs
-        elif isinstance(thing, Atom):
+        if isinstance(thing, Atom):
             pass
         elif isinstance(thing, Definition):
             thing = thing.atom
@@ -674,31 +682,30 @@ class LiveDefinitions:
 
         if isinstance(thing, Register):
             return self.get_register_definitions(thing.reg_offset, thing.size)
-        elif isinstance(thing, MemoryLocation):
+        if isinstance(thing, MemoryLocation):
             if isinstance(thing.addr, SpOffset):
                 return self.get_stack_definitions(thing.addr.offset, thing.size)
-            elif isinstance(thing.addr, HeapAddress):
+            if isinstance(thing.addr, HeapAddress):
+                if isinstance(thing.addr.value, Undefined):
+                    return set()
                 return self.get_heap_definitions(thing.addr.value, size=thing.size)
-            elif isinstance(thing.addr, int):
+            if isinstance(thing.addr, int):
                 return self.get_memory_definitions(thing.addr, thing.size)
-            else:
-                return set()
-        elif isinstance(thing, Tmp):
+            return set()
+        if isinstance(thing, Tmp):
             return self.get_tmp_definitions(thing.tmp_idx)
-        else:
-            defs = set()
-            for mvs in self.others.get(thing, {}).values():
-                for mv in mvs:
-                    defs |= self.get_definitions(mv)
-            return defs
+        defs = set()
+        mv = self.others.get(thing, None)
+        if mv is not None:
+            defs |= self.get_definitions(mv)
+        return defs
 
-    def get_tmp_definitions(self, tmp_idx: int) -> Set[Definition]:
+    def get_tmp_definitions(self, tmp_idx: int) -> set[Definition]:
         if tmp_idx in self.tmps:
             return self.tmps[tmp_idx]
-        else:
-            return set()
+        return set()
 
-    def get_register_definitions(self, reg_offset: int, size: int) -> Set[Definition]:
+    def get_register_definitions(self, reg_offset: int, size: int) -> set[Definition]:
         try:
             annotations = self.registers.load_annotations(reg_offset, size)
         except SimMemoryMissingError as ex:
@@ -710,14 +717,14 @@ class LiveDefinitions:
 
         return LiveDefinitions.extract_defs_from_annotations(annotations)
 
-    def get_stack_values(self, stack_offset: int, size: int, endness: str) -> Optional[MultiValues]:
+    def get_stack_values(self, stack_offset: int, size: int, endness: str) -> MultiValues | None:
         stack_addr = self.stack_offset_to_stack_addr(stack_offset)
         try:
             return self.stack.load(stack_addr, size=size, endness=endness)
         except SimMemoryMissingError:
             return None
 
-    def get_stack_definitions(self, stack_offset: int, size: int) -> Set[Definition]:
+    def get_stack_definitions(self, stack_offset: int, size: int) -> set[Definition]:
         try:
             stack_addr = self.stack_offset_to_stack_addr(stack_offset)
             annotations = self.stack.load_annotations(stack_addr, size)
@@ -726,7 +733,7 @@ class LiveDefinitions:
 
         return LiveDefinitions.extract_defs_from_annotations(annotations)
 
-    def get_heap_definitions(self, heap_addr: int, size: int) -> Set[Definition]:
+    def get_heap_definitions(self, heap_addr: int, size: int) -> set[Definition]:
         try:
             annotations = self.heap.load_annotations(heap_addr, size)
         except SimMemoryMissingError:
@@ -734,7 +741,7 @@ class LiveDefinitions:
 
         return LiveDefinitions.extract_defs_from_annotations(annotations)
 
-    def get_memory_definitions(self, addr: int, size: int) -> Set[Definition]:
+    def get_memory_definitions(self, addr: int, size: int) -> set[Definition]:
         try:
             annotations = self.memory.load_annotations(addr, size)
         except SimMemoryMissingError:
@@ -742,74 +749,9 @@ class LiveDefinitions:
 
         return LiveDefinitions.extract_defs_from_annotations(annotations)
 
-    @deprecated("get_definitions")
-    def get_definitions_from_atoms(self, atoms: Iterable[Atom]) -> Iterable[Definition]:
-        result = set()
-        for atom in atoms:
-            result |= self.get_definitions(atom)
-        return result
-
-    @deprecated("get_values")
-    def get_value_from_definition(self, definition: Definition) -> Optional[MultiValues]:
-        return self.get_value_from_atom(definition.atom)
-
-    @deprecated("get_one_value")
-    def get_one_value_from_definition(self, definition: Definition) -> Optional[claripy.ast.bv.BV]:
-        return self.get_one_value_from_atom(definition.atom)
-
-    @deprecated("get_concrete_value")
-    def get_concrete_value_from_definition(self, definition: Definition) -> Optional[int]:
-        return self.get_concrete_value_from_atom(definition.atom)
-
-    @deprecated("get_values")
-    def get_value_from_atom(self, atom: Atom) -> Optional[MultiValues]:
-        if isinstance(atom, Register):
-            try:
-                return self.registers.load(atom.reg_offset, size=atom.size)
-            except SimMemoryMissingError:
-                return None
-        elif isinstance(atom, MemoryLocation):
-            if isinstance(atom.addr, SpOffset):
-                stack_addr = self.stack_offset_to_stack_addr(atom.addr.offset)
-                try:
-                    return self.stack.load(stack_addr, size=atom.size, endness=atom.endness)
-                except SimMemoryMissingError:
-                    return None
-            elif isinstance(atom.addr, HeapAddress):
-                try:
-                    return self.heap.load(atom.addr.value, size=atom.size, endness=atom.endness)
-                except SimMemoryMissingError:
-                    return None
-            elif isinstance(atom.addr, int):
-                try:
-                    return self.memory.load(atom.addr, size=atom.size, endness=atom.endness)
-                except SimMemoryMissingError:
-                    return None
-            else:
-                # ignore RegisterOffset
-                return None
-        else:
-            return None
-
-    @deprecated("get_one_value")
-    def get_one_value_from_atom(self, atom: Atom) -> Optional[claripy.ast.bv.BV]:
-        r = self.get_value_from_atom(atom)
-        if r is None:
-            return None
-        return r.one_value()
-
-    @deprecated("get_concrete_value")
-    def get_concrete_value_from_atom(self, atom: Atom) -> Optional[int]:
-        r = self.get_one_value_from_atom(atom)
-        if r is None:
-            return None
-        if r.symbolic:
-            return None
-        return r.concrete_value
-
     def get_values(
-        self, spec: Union[Atom, Definition[Atom], Iterable[Atom], Iterable[Definition[Atom]]]
-    ) -> Optional[MultiValues]:
+        self, spec: A | Definition[A] | Iterable[A] | Iterable[Definition[A]], endness: archinfo.Endness | None = None
+    ) -> MultiValues | None:
         if isinstance(spec, Definition):
             atom = spec.atom
         elif isinstance(spec, Atom):
@@ -817,13 +759,10 @@ class LiveDefinitions:
         else:
             result = None
             for atom in spec:
-                r = self.get_values(atom)
+                r = self.get_values(atom, endness)
                 if r is None:
                     continue
-                if result is None:
-                    result = r
-                else:
-                    result = result.merge(r)
+                result = r if result is None else result.merge(r)
             return result
 
         if isinstance(atom, Register):
@@ -831,32 +770,33 @@ class LiveDefinitions:
                 return self.registers.load(atom.reg_offset, size=atom.size)
             except SimMemoryMissingError:
                 return None
-        elif isinstance(atom, MemoryLocation):
+
+        if isinstance(atom, MemoryLocation):
+            endness = endness or atom.endness
             if isinstance(atom.addr, SpOffset):
                 stack_addr = self.stack_offset_to_stack_addr(atom.addr.offset)
                 try:
-                    return self.stack.load(stack_addr, size=atom.size, endness=atom.endness)
+                    return self.stack.load(stack_addr, size=atom.size, endness=endness)
                 except SimMemoryMissingError:
                     return None
             elif isinstance(atom.addr, HeapAddress):
                 try:
-                    return self.heap.load(atom.addr.value, size=atom.size, endness=atom.endness)
+                    return self.heap.load(atom.addr.value, size=atom.size, endness=endness)
                 except SimMemoryMissingError:
                     return None
             elif isinstance(atom.addr, int):
                 try:
-                    return self.memory.load(atom.addr, size=atom.size, endness=atom.endness)
+                    return self.memory.load(atom.addr, size=atom.size, endness=endness)
                 except SimMemoryMissingError:
                     pass
                 if self.project is not None:
                     try:
                         bytestring = self.project.loader.memory.load(atom.addr, atom.size)
-                        if atom.endness == archinfo.Endness.LE:
+                        if endness == archinfo.Endness.LE:
                             bytestring = bytes(reversed(bytestring))
-                        mv = MultiValues(
+                        return MultiValues(
                             self.annotate_with_def(claripy.BVV(bytestring), Definition(atom, ExternalCodeLocation()))
                         )
-                        return mv
                     except KeyError:
                         pass
                 return None
@@ -868,9 +808,9 @@ class LiveDefinitions:
 
     def get_one_value(
         self,
-        spec: Union[Atom, Definition, Iterable[Atom], Iterable[Definition[Atom]]],
+        spec: A | Definition[A] | Iterable[A] | Iterable[Definition[A]],
         strip_annotations: bool = False,
-    ) -> Optional[claripy.ast.bv.BV]:
+    ) -> claripy.ast.bv.BV | None:
         r = self.get_values(spec)
         if r is None:
             return None
@@ -878,21 +818,19 @@ class LiveDefinitions:
 
     @overload
     def get_concrete_value(
-        self, spec: Union[Atom, Definition[Atom], Iterable[Atom], Iterable[Definition[Atom]]], cast_to: Type[int] = ...
-    ) -> Optional[int]: ...
+        self, spec: A | Definition[A] | Iterable[A] | Iterable[Definition[A]], cast_to: type[int]
+    ) -> int | None: ...
 
     @overload
     def get_concrete_value(
-        self,
-        spec: Union[Atom, Definition[Atom], Iterable[Atom], Iterable[Definition[Atom]]],
-        cast_to: Type[bytes] = ...,
-    ) -> Optional[bytes]: ...
+        self, spec: A | Definition[A] | Iterable[A] | Iterable[Definition[A]], cast_to: type[bytes]
+    ) -> bytes | None: ...
 
     def get_concrete_value(
         self,
-        spec: Union[Atom, Definition[Atom], Iterable[Atom], Iterable[Definition[Atom]]],
-        cast_to: Union[Type[int], Type[bytes]] = int,
-    ) -> Union[int, bytes, None]:
+        spec: A | Definition[A] | Iterable[A] | Iterable[Definition[A]],
+        cast_to: type[int] | type[bytes] = int,
+    ) -> int | bytes | None:
         r = self.get_one_value(spec, strip_annotations=True)
         if r is None:
             return None
@@ -903,7 +841,7 @@ class LiveDefinitions:
             return result.to_bytes(len(r) // 8, "big")
         return result
 
-    def add_register_use(self, reg_offset: int, size: int, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_register_use(self, reg_offset: int, size: int, code_loc: CodeLocation, expr: Any | None = None) -> None:
         # get all current definitions
         try:
             mvs: MultiValues = self.registers.load(reg_offset, size=size)
@@ -915,42 +853,42 @@ class LiveDefinitions:
                 for def_ in self.extract_defs(v):
                     self.add_register_use_by_def(def_, code_loc, expr=expr)
 
-    def add_register_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_register_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Any | None = None) -> None:
         self.register_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
-    def add_stack_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_stack_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Any | None = None) -> None:
         if not isinstance(atom.addr, SpOffset):
-            raise TypeError("Atom %r is not a stack location atom." % atom)
+            raise TypeError(f"Atom {atom!r} is not a stack location atom.")
 
         for current_def in self.get_definitions(atom):
             self.add_stack_use_by_def(current_def, code_loc, expr=expr)
 
-    def add_stack_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_stack_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Any | None = None) -> None:
         self.stack_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
-    def add_heap_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_heap_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Any | None = None) -> None:
         if not isinstance(atom.addr, HeapAddress):
-            raise TypeError("Atom %r is not a heap location atom." % atom)
+            raise TypeError(f"Atom {atom!r} is not a heap location atom.")
 
         current_defs = self.get_definitions(atom)
 
         for current_def in current_defs:
             self.add_heap_use_by_def(current_def, code_loc, expr=expr)
 
-    def add_heap_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_heap_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Any | None = None) -> None:
         self.heap_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
-    def add_memory_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_memory_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Any | None = None) -> None:
         # get all current definitions
-        current_defs: Set[Definition] = self.get_definitions(atom)
+        current_defs: set[Definition] = self.get_definitions(atom)
 
         for current_def in current_defs:
             self.add_memory_use_by_def(current_def, code_loc, expr=expr)
 
-    def add_memory_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any] = None) -> None:
+    def add_memory_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Any | None = None) -> None:
         self.memory_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
@@ -969,7 +907,7 @@ class LiveDefinitions:
 
     def add_tmp_use_by_def(self, def_: Definition, code_loc: CodeLocation) -> None:
         if not isinstance(def_.atom, Tmp):
-            raise TypeError("Atom %r is not a Tmp atom." % def_.atom)
+            raise TypeError(f"Atom {def_.atom!r} is not a Tmp atom.")
 
         self.tmp_uses[def_.atom.tmp_idx].add(code_loc)
         self.uses_by_codeloc[code_loc].add(def_)
@@ -977,22 +915,22 @@ class LiveDefinitions:
     @overload
     def deref(
         self,
-        pointer: Union[MultiValues, Atom, Definition, Iterable[Atom], Iterable[Definition]],
-        size: Union[int, DerefSize],
+        pointer: MultiValues[claripy.ast.BV] | Atom | Definition[Atom] | Iterable[Atom] | Iterable[Definition[Atom]],
+        size: int | DerefSize,
         endness: archinfo.Endness = ...,
-    ) -> Set[MemoryLocation]: ...
+    ) -> set[MemoryLocation]: ...
 
     @overload
     def deref(
         self,
-        pointer: Union[int, claripy.ast.BV, HeapAddress, SpOffset],
-        size: Union[int, DerefSize],
+        pointer: int | claripy.ast.BV | HeapAddress | SpOffset,
+        size: int | DerefSize,
         endness: archinfo.Endness = ...,
-    ) -> Optional[MemoryLocation]: ...
+    ) -> MemoryLocation | None: ...
 
     def deref(self, pointer, size, endness=archinfo.Endness.BE):
         if isinstance(pointer, (Atom, Definition)):
-            pointer = self.get_values(pointer)
+            pointer = self.get_values(pointer, self.arch.memory_endness)
             if pointer is None:
                 return set()
 
@@ -1027,17 +965,26 @@ class LiveDefinitions:
                 if heap_offset is not None:
                     addr = HeapAddress(heap_offset)
                 elif pointer.op == "BVV":
-                    addr = pointer.args[0]
+                    addr = cast(int, pointer.args[0])
                 else:
                     # cannot resolve
                     return None
 
         if isinstance(size, DerefSize):
-            assert size == DerefSize.NULL_TERMINATE
-            for sz in range(4096):  # arbitrary
+            MAX_SIZE = 4096
+            for sz in range(MAX_SIZE):  # arbitrary
                 # truly evil that this is an abstraction we have to contend with
                 mv = self.get_values(MemoryLocation(addr + sz, 1, endness))
-                if mv is not None and 0 in mv and any(one.op == "BVV" and one.args[0] == 0 for one in mv[0]):
+
+                if mv is None or 0 not in mv:
+                    # If the sz is 0, then we found nothing and yolo the size
+                    # if the mv is None, then we went past the indexed memory and should use sz
+                    # (mostly for the BVS case where no null terminator is guaranteed)
+                    size = MAX_SIZE if sz == 0 else sz
+                    break
+
+                if any(one.op == "BVV" and one.args[0] == 0 for one in mv[0]):
+                    # We found a null terminator
                     size = sz + 1
                     break
             else:
@@ -1045,7 +992,7 @@ class LiveDefinitions:
                     "Could not resolve cstring dereference at %s to a concrete size",
                     hex(addr) if isinstance(addr, int) else addr,
                 )
-                size = 4096
+                size = MAX_SIZE
 
         return MemoryLocation(addr, size, endness)
 
@@ -1054,15 +1001,15 @@ class LiveDefinitions:
         return "heap_base" in addr.variables
 
     @staticmethod
-    def get_heap_offset(addr: claripy.ast.Base) -> Optional[int]:
+    def get_heap_offset(addr: claripy.ast.Base) -> int | None:
         if "heap_base" in addr.variables:
             if addr.op == "BVS":
                 return 0
-            elif addr.op == "__add__" and len(addr.args) == 2 and addr.args[1].op == "BVV":
-                return addr.args[1].concrete_value
+            if addr.op == "__add__" and len(addr.args) == 2 and cast(claripy.ast.BV, addr.args[1]).op == "BVV":
+                return cast(claripy.ast.BV, addr.args[1]).concrete_value
         return None
 
-    def heap_address(self, offset: Union[int, HeapAddress]) -> claripy.ast.BV:
+    def heap_address(self, offset: int | HeapAddress) -> claripy.ast.BV:
         if isinstance(offset, HeapAddress):
             if not isinstance(offset.value, int):
                 raise TypeError("TODO: what's this? contact @rhelmot")
