@@ -308,9 +308,12 @@ class Project:
                 l.info("There are no simprocedures for missing library %s :(", lib_name)
                 if lib_name.startswith("api-ms-win-"):
                     missing_wincore_dlls = True
-        if missing_wincore_dlls and "kernel32.dll" not in self.loader.missing_dependencies:
-            # some of the missing api-ms-win-*.dll libraries are actually provided by kernel32.dll
-            missing_libs.extend(SIM_LIBRARIES["kernel32.dll"])
+        if missing_wincore_dlls:
+            # some of the missing api-ms-win-*.dll libraries are actually provided by kernel32.dll and advapi32.dll
+            if "kernel32.dll" not in self.loader.missing_dependencies:
+                missing_libs.extend(SIM_LIBRARIES["kernel32.dll"])
+            if "advapi32.dll" not in self.loader.missing_dependencies:
+                missing_libs.extend(SIM_LIBRARIES["advapi32.dll"])
 
         # additionally provide libraries we _have_ loaded as a fallback fallback
         # this helps in the case that e.g. CLE picked up a linux arm libc to satisfy an android arm binary
