@@ -1,5 +1,6 @@
 # This module contains the analysis options.
 # All variables with names of all caps will be registered as a state option to SimStateOptions.
+from __future__ import annotations
 
 import string
 from .sim_state_options import SimStateOptions
@@ -44,12 +45,9 @@ NO_SYMBOLIC_JUMP_RESOLUTION = "NO_SYMBOLIC_JUMP_RESOLUTION"
 # This option prevents angr from doing hundreds of constraint solves when it hits a symbolic syscall
 NO_SYMBOLIC_SYSCALL_RESOLUTION = "NO_SYMBOLIC_SYSCALL_RESOLUTION"
 
-# The absense of this option causes the analysis to avoid reasoning about most symbolic values.
+# The absence of this option causes the analysis to avoid reasoning about most symbolic values.
 SYMBOLIC = "SYMBOLIC"
 
-# This variable causes claripy to use a string solver (CVC4)
-STRINGS_ANALYSIS = "STRINGS_ANALYSIS"
-
 # Generate symbolic values for non-existent values. The absence of this option causes Unconstrained() to return default
 # concrete values (like 0)
 SYMBOLIC_INITIAL_VALUES = "SYMBOLIC_INITIAL_VALUES"
@@ -73,9 +71,12 @@ CONCRETIZE_SYMBOLIC_FILE_READ_SIZES = "CONCRETIZE_SYMBOLIC_FILE_READ_SIZES"
 FILES_HAVE_EOF = "FILES_HAVE_EOF"
 UNKNOWN_FILES_HAVE_EOF = FILES_HAVE_EOF
 
-# Attempting to open an unkown file will result in creating it with a symbolic length
+# Attempting to open an unknown file will result in creating it with a symbolic length
 ALL_FILES_EXIST = "ALL_FILES_EXIST"
 
+# Unknown files might or might not exist
+ANY_FILE_MIGHT_EXIST = "ANY_FILE_MIGHT_EXIST"
+
 # Reads from devices will have a symbolic size
 SHORT_READS = "SHORT_READS"
 
@@ -231,10 +232,6 @@ KEEP_IP_SYMBOLIC = "KEEP_IP_SYMBOLIC"
 # Do not try to concretize a symbolic IP. With this option, all states with symbolic IPs will be seen as unconstrained.
 NO_IP_CONCRETIZATION = "NO_IP_CONCRETIZATION"
 
-# Do not union values from different locations when reading from the memory for a reduced loss in precision
-# It is only applied to SimAbstractMemory
-KEEP_MEMORY_READS_DISCRETE = "KEEP_MEMORY_READS_DISCRETE"
-
 # Raise a SimSegfaultError on illegal memory accesses
 STRICT_PAGE_ACCESS = "STRICT_PAGE_ACCESS"
 
@@ -312,15 +309,9 @@ CGC_ENFORCE_FD = "CGC_ENFORCE_FD"
 CGC_NON_BLOCKING_FDS = "CGC_NON_BLOCKING_FDS"
 
 # Allows memory breakpoints to get more accurate sizes in case of reading large chunks
-# Sacrafice performance for more fine tune memory read size
+# Sacrifice performance for more fine tune memory read size
 MEMORY_CHUNK_INDIVIDUAL_READS = "MEMORY_CHUNK_INDIVIDUAL_READS"
 
-# Synchronize memory mapping reported by angr with the concrete process.
-SYMBION_SYNC_CLE = "SYMBION_SYNC_CLE"
-# Removes stubs SimProc on synchronization with concrete process.
-# We will execute SimProc for functions for which we have one, and the real function for the one we have not.
-SYMBION_KEEP_STUBS_ON_SYNC = "SYMBION_KEEP_STUBS_ON_SYNC"
-
 # Activate the heuristic that tries to understand if an unknown method (e. g. library call) gets or sets an
 # attribute inside the object. Turn on this option to have a better approximation of the internal structure of the
 # symbolic object.
@@ -334,7 +325,7 @@ JAVA_TRACK_ATTRIBUTES = "JAVA_TRACK_ATTRIBUTES"
 
 _g = globals().copy()
 for k, v in _g.items():
-    if all([char in string.ascii_uppercase + "_" + string.digits for char in k]) and type(v) is str:
+    if all(char in string.ascii_uppercase + "_" + string.digits for char in k) and type(v) is str:
         if k in (
             "UNKNOWN_FILES_HAVE_EOF",
             "CGC_ZERO_FILL_UNCONSTRAINED_MEMORY",
